//
//  tsprintf5.c
//
// stdio.h "sprintf" like print data formatter
//	buf: memory buffer
//	fmt: %[<Flags>...][<Width>[.<Precision>]][<Modifier>]<Conversions>
//		Flags: {-|+| |#|0}
//		Width: 1~255 Fixed column (not minimum width)
//		Precision: 0~255 N/A
//		Modifier: {hh|h|l|ll|j|z|t}
//		Conversions: {d|i|u|x|o|c|s|%}
//	v[1-5]: variable address 
//		%s is not "const" area
//  Copyright (C) 2009 Toyohiko Togashi tog001@nifty.com
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//	Update history
//	---------- ----- -------------------------------------------
//	2009.08.02 v0.0  First cording for ImageCraft C Compiler
//	2009.12.18 v0.0  bug fix, %hhd unsign
//

#include <string.h>
#include <stdlib.h>
void tsprintf5(char *buf, const char *fmt,
	void *v1, void *v2, void *v3, void *v4, void *v5) {
	signed char n;						// argument#

	n = 0;
	while(*fmt != '\0') {

		if (*fmt != '%') {
			*buf++ = *fmt++;
		} else {
			char a;						// align
			char s;						// sign char
			char z;						// padding
			signed char w;				// width
			signed char p;				// precision
			char f;						// format char
			char t;						// data type
			char pf[5];					// piece of fmt
			void *arg;					// argument

			fmt++;
			a = 0;
			s = '\0';
			z = ' ';
			w = 0;
			p = 0;
			f = '\0';
			t = '\0';

			//
			// Format analyze
			//

			// Get flag
			while(*fmt != '\0') {
				char c;

				c = *fmt;
				switch(c) {
				case '-':				// reverse align
					a = c;
					break;
				case '+':				// force sign +/-
					s = c;
					break;
				case ' ':               // force sign  /-
					s = c;
					break;
				case '#':				// N/A
					break;
				case '0':				// zero padding
					z = c;
					break;
				default:
					c = '\0';
					break;
				}
				if (c == '\0') {
					break;
				}
				fmt++;
			}

			// get fixed field width
			cstrncpy(pf, fmt, sizeof(pf)-1);
			pf[sizeof(pf)-1] = '\0';
			w = atoi(pf);
			while(isdigit(*fmt)) fmt++;

			// get precision field
			if (*fmt == '.') {
				fmt++;
				cstrncpy(pf, fmt, sizeof(pf)-1);
				pf[sizeof(pf)-1] = '\0';
				p = atoi(pf);
				while(isdigit(*fmt)) fmt++;
			}

			// get format charactor or data type
			while(*fmt != '\0') {
				f = *fmt++;
				if ((f == 'h') || (f == 'l') || (f == 'j') || (f == 'z') || (f == 't')) {
					if (t != '\0') {
						t = toupper(t);
					} else {
						t = f;
					}
				} else {
					break;
				}
			}

			//
			// set argument addr
			//
			n++;
			if (n > 5) {				// Argument over
				break;
			}
			switch(n) {
			case 1:
				arg = v1;
				break;
			case 2:
				arg = v2;
				break;
			case 3:
				arg = v3;
				break;
			case 4:
				arg = v4;
				break;
			default:
				arg = v5;
				break;
			}

			//
			// Format out
			//
			switch(f) {
			case '%':
				*buf++ = f;
				break;
			case 'c':
				*buf++ = *((int *)arg);
				break;
			case 'd':
			case 'i':
			case 'u':
			case 'o':
			case 'x':
			case 's':
				{
					signed char l;		// data length
					char tb[16];		// number->str
					char *ts;			// target str

					if (f == 's') {
						ts = (char *)arg;
					} else {
						unsigned char b;

						// radix
						switch(f) {
						case 'x':
							b = 16;
							break;
						case 'o':
							b = 8;
							break;
						default:
							b = 10;
							break;
						}

						// signed number
						if ((f == 'd') || (f == 'i')) {
							signed long d;

							switch(t) {
							case 'h':
								d = *((short *)arg);
								break;
							case 'H':
//								d = *((char *)arg);				// DEL 2009.12.18
								d = *((signed char *)arg);		// ADD 2009.12.18
								break;
							case 'l':
							case 'L':
								d = *((long *)arg);
								break;
							default:
								d = *((int *)arg);
								break;
							}

							
							if ((s == '\0') || (d < 0)) {
								ltoa(tb, d, b);
							} else {	// +/-option
								tb[0] = s;
								ltoa(&tb[1], d, b);
							}

						// unsigned number
						} else {
							unsigned long u;

							switch(t) {
							case 'h':
								u = *((unsigned short *)arg);
								break;
							case 'H':
								u = *((unsigned char *)arg);
								break;
							case 'l':
							case 'L':
								u = *((unsigned long *)arg);
								break;
							default:
								u = *((unsigned int *)arg);
								break;
							}

							ultoa(tb, u, b);

						}

						ts = tb;
						a = !a;
					}

					l = strlen(ts);
					if (w == 0) {
						w = l;
					}

					if (a) {			// Right align
						signed char i;
					
						i = w - l;
						if (i < 0) {	// s is longer
							ts += (-i + 1);	// skip char's
							*buf++ = '~';	// overflow marking
							--w;
							l = w;
						}
						for(; i > 0; --i, --w) {
							*buf++ = z;	// Before padding
						}
					}
					for(; *ts != '\0' && w > 0; ts++, --w) {
						*buf++ = *ts;
					}
					if (*ts != '\0') {
						*(buf-1) = '~';	// overflow marking
					}
					for(; w > 0; --w) {	// Left align
						*buf++ = z;		// After padding
					}
				}
				break;
			default:
				*buf++ = f;
				break;
			}
		}
	}
	*buf = '\0';
	return;
}
